import { createStore } from "solid-js/store";
import {
    type InferOutput,
    literal,
    pipe,
    safeParse,
    strictObject,
    string,
    union,
    uuid,
} from "valibot";
import { base } from "../network";
import { createContext, onCleanup } from "solid-js";

const processingImagesValidator = strictObject({
    ImageID: pipe(string(), uuid()),
    Status: union([literal("in-progress"), literal("complete")]),
});

type NotificationState = {
    ProcessingImages: Record<
        string,
        InferOutput<typeof processingImagesValidator>["Status"] | undefined
    >;
};

export const Notifications = (onCompleteImage: () => void) => {
    const [state, setState] = createStore<NotificationState>({
        ProcessingImages: {},
    });

    const access = localStorage.getItem("access");
    if (access == null) {
        throw new Error("Access token not defined");
    }

    const dataEventListener = (e: MessageEvent<unknown>) => {
        if (typeof e.data !== "string") {
            console.error("Error type is not string");
            return;
        }

        let jsonData: object = {};
        try {
            jsonData = JSON.parse(e.data);
        } catch (e) {
            console.error(e);
            return;
        }

        const processingImage = safeParse(processingImagesValidator, jsonData);
        if (!processingImage.success) {
            console.error("Processing image could not be parsed.", e.data);
            return;
        }

        const { ImageID, Status } = processingImage.output;

        if (state.ProcessingImages[ImageID] != null) {
            const localImageStatus = state.ProcessingImages[ImageID];
            if (localImageStatus !== "in-progress" || Status !== "complete") {
                console.error(
                    "Invalid state, an image present must always be in process",
                );
                return;
            }

            setState("ProcessingImages", ImageID, undefined);
            onCompleteImage();
        } else {
            if (Status !== "in-progress") {
                console.error(
                    "Invalid state, at this point a new image that isn't present should be in-progress",
                );
                return;
            }

            setState("ProcessingImages", ImageID, Status);
        }
    };

    const events = new EventSource(`${base}/notifications?token=${access}`);

    events.addEventListener("data", dataEventListener);

    onCleanup(() => {
        events.removeEventListener("data", dataEventListener);
        events.close();
    });

    return {
        state,
    };
};

export const NotificationsContext =
    createContext<ReturnType<typeof Notifications>>();
